//	PLATFORMS:	Windows 95, Windows NT, Win32s
//
//

#include "runit.h"


ATOM MyRegisterClass(CONST WNDCLASS*);
BOOL InitApplication(HINSTANCE);
BOOL InitInstance(HINSTANCE, int);

extern BOOL WriteLog(char *szLogfile,char *szMsg);
extern LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
extern LRESULT CALLBACK MenuWndProc(HWND, UINT, WPARAM, LPARAM);
extern LRESULT CALLBACK QuickRefProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
extern LRESULT CALLBACK SetupProc(HWND, UINT, WPARAM, LPARAM);
extern _error(int iSeverity,char* szText, char* szFunction);
extern BOOL CheckIniFile(char* lpCmdLine,char* szMainIni);
extern BOOL ReadFromIni(char *,BOOL,int,BOOL);
extern BOOL DoHooking();
extern BOOL RedrawWnd(HWND);
extern BOOL ParseMenu(int);
extern BOOL DeleteSet(struct _RUNit*, struct _SET*, int);
extern struct _SET Set[8];
extern struct _SET Set_tmp[8];

extern char szAppName[]; // The name of this application
extern char szTitle[]; // The title bar text
extern char szMainIni[MAX_PATH];		// the full path to the ini file
extern HWND hwndMain;
extern HHOOK hRunitHook;			// the hook
extern HINSTANCE hinstDll;			// the hook-dll
extern char szDLLPath[MAX_PATH];	// the full path to the hook-dll
extern NOTIFYICONDATA nid;			// for the tray icon
extern int 	iScreenWidth;
extern int	iScreenHeight;
extern HINSTANCE hInst;
extern struct RUNIT_OPTIONS opt;	//options from the options dialog
extern struct _RUNit RUNit;		//internal values (have to do with the menu)
extern DWORD StayTopMostThreadProc(LPVOID lpParms);
extern BOOL MakeIniBackup(char *szIni);
extern BOOL bDialogIsActive;
extern BOOL bShouldExit=FALSE;
extern HMENU hAppMenu[8];



//  FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)
//
//
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	MSG msg;
	int i;

	//clear the SET struct
	ZeroMemory(&Set,sizeof(struct _SET)*8);
	//allocate mem for  APP2RUN structs
	for (i=0;i<8;i++){
		Set[i].ap=HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,sizeof(struct APP2RUN));
		Set_tmp[i].ap=HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,sizeof(struct APP2RUN));
	}
	if( (!Set[0].ap)||(!Set_tmp[0].ap) )  //not very probably, but i dont wanna risk an exception..
	{
		_error(FATAL,"Memory Allocation Error","WinMain()");
		return -1;
	}

	InitCommonControls();
	if(!LoadLibrary("RICHED32.DLL"))
	{
		_error(CONTINUE,"Failed to load Library RICHED32.DLL","WinMain()");
	}


	// check if an INI file has been specified and if it is OK
	if(!CheckIniFile(lpCmdLine,szMainIni))
		return -1;


	if (!hPrevInstance) {
		// Perform instance initialization:
		if (!InitApplication(hInstance)) {
			return (FALSE);
		}
	}
	
	// Perform application initialization:
	if (!InitInstance(hInstance, nCmdShow)) {
		return (FALSE);
	}

	__try{
		// Main message loop:
		while (GetMessage(&msg, NULL, 0, 0)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}

		UnhookWindowsHookEx(hRunitHook);
		Sleep(0);
		FreeLibrary(hinstDll);
		Sleep(0);
		DeleteFile(szDLLPath);
	}
	__except(UnhookWindowsHookEx(hRunitHook),Shell_NotifyIcon(NIM_DELETE,&nid),MessageBox(NULL,"An exception occured in RUNit...\nthe program will be killed by windows.\n\nI'm really sorrry about that...\n\nYou can help me to avoid this problem in future releases by sending me the information provided in the following dialog\n(you have to press the 'Details-button' to get the important information).","Sorry...",MB_OK|MB_ICONERROR), EXCEPTION_CONTINUE_SEARCH){
	}

	return (msg.wParam);

}




//  FUNCTION: MyRegisterClass(CONST WNDCLASS*)
//
//
ATOM MyRegisterClass(CONST WNDCLASS *lpwc)
{
	HANDLE  hMod;
	FARPROC proc;
	WNDCLASSEX wcex;

	hMod = GetModuleHandle ("USER32");
	if (hMod != NULL) {

	#if defined (UNICODE)
		proc = GetProcAddress (hMod, "RegisterClassExW");
	#else
		proc = GetProcAddress (hMod, "RegisterClassExA");
	#endif

		if (proc != NULL) {

			wcex.style         = lpwc->style;
			wcex.lpfnWndProc   = lpwc->lpfnWndProc;
			wcex.cbClsExtra    = lpwc->cbClsExtra;
			wcex.cbWndExtra    = lpwc->cbWndExtra;
			wcex.hInstance     = lpwc->hInstance;
			wcex.hIcon         = lpwc->hIcon;
			wcex.hCursor       = lpwc->hCursor;
			wcex.hbrBackground = lpwc->hbrBackground;
   			wcex.lpszMenuName  = lpwc->lpszMenuName;
			wcex.lpszClassName = lpwc->lpszClassName;

			// Added elements for Windows 95:
			wcex.cbSize = sizeof(WNDCLASSEX);
			wcex.hIconSm = LoadIcon(wcex.hInstance, "SMALL");
			
			return (*proc)(&wcex);//return RegisterClassEx(&wcex);
		}
	}
	return (RegisterClass(lpwc));
}


//
//  FUNCTION: InitApplication(HANDLE)
//
//
BOOL InitApplication(HINSTANCE hInstance)
{
    WNDCLASS  wcRoot;
    WNDCLASS  wcMenu;
	
    if (FindWindow (szAppName, NULL)){
		MessageBox(NULL,"An instance of RUNit is already running...\n\nRUNit wants to be unique.","RUNit",MB_OK|MB_ICONINFORMATION);
		return FALSE;
	}



	//fuer menu
	//RUNit.iTooLarge=0;

	iScreenWidth=GetSystemMetrics(SM_CXSCREEN);
	iScreenHeight=GetSystemMetrics(SM_CYSCREEN);


    wcRoot.style         = CS_HREDRAW | CS_VREDRAW;
    wcRoot.lpfnWndProc   = (WNDPROC)WndProc;
    wcRoot.cbClsExtra    = 0;
    wcRoot.cbWndExtra    = 0;
    wcRoot.hInstance     = hInstance;
    wcRoot.hIcon         = LoadIcon (hInstance, szAppName);
    wcRoot.hCursor       = LoadCursor(NULL, IDC_ARROW);
	wcRoot.hbrBackground = GetSysColorBrush(COLOR_MENU);
	wcRoot.lpszMenuName  = NULL;//"WIN95";
    wcRoot.lpszClassName = szAppName;

    wcMenu.style         = CS_HREDRAW | CS_VREDRAW;
    wcMenu.lpfnWndProc   = (WNDPROC)MenuWndProc;
    wcMenu.cbClsExtra    = 0;
    wcMenu.cbWndExtra    = 0;
    wcMenu.hInstance     = hInstance;
    wcMenu.hIcon         = LoadIcon (hInstance, szAppName);
    wcMenu.hCursor       = LoadCursor(NULL, IDC_ARROW);
	wcMenu.hbrBackground = GetSysColorBrush(COLOR_MENU);
	wcMenu.lpszMenuName  = NULL;//"WIN95";
    wcMenu.lpszClassName = "RUNitMenu";

        // Register the window class and return success/failure code.
        if (IS_WIN95)
		{
			return (MyRegisterClass(&wcMenu)&&MyRegisterClass(&wcRoot));
        }
		else
		{
			return (RegisterClass(&wcRoot)&&RegisterClass(&wcMenu));
        }
}



//
//   FUNCTION: InitInstance(HANDLE, int)
//
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	HWND hWnd;
	char szDrive[MAX_PATH];
	char szDir[MAX_PATH];
	char szFile[MAX_PATH];
	char szExt[MAX_PATH];
	char sz[MAX_PATH];
	char szSetError[MAX_PATH*2];
	HRSRC hrsrc;
	HGLOBAL hglob;
	LPVOID lpvoid;
	DWORD ressize;
	DWORD bwritten;
	HANDLE hFile;
	LPVOID lpMsgBuf;
	int iret=0;
	//DWORD tid;
	int i,iFailed;
	BOOL bOK;


	hInst = hInstance; // Store instance handle in our global variable


	//Xtract the dll
	//hrsrc=FindResource(hInst,MAKEINTRESOURCE(IDR_DATA1),RT_RCDATA);
	//hglob=LoadResource(NULL,hrsrc);
	//ressize=SizeofResource(hInst,hrsrc);
	//lpvoid=LockResource(hglob);

	//2do: check auf \ !!!
	//GetModuleFileName(NULL,szDLLPath,sizeof(szDLLPath));
	//_splitpath(szDLLPath,szDrive,szDir,szFile,szExt);
	//strcpy(szDLLPath,szDrive);
	//strcat(szDLLPath,szDir);
	//strcat(szDLLPath,"runit.lex");
	//hFile=CreateFile(szDLLPath,GENERIC_READ|GENERIC_WRITE,0,0,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,0);
	//WriteFile(hFile,lpvoid,ressize,&bwritten,NULL);
	//CloseHandle(hFile);

	//hinstDll=LoadLibrary(szDLLPath);
	hinstDll=LoadLibrary("runitlib.dll");
	SetRUNITHook=(SETRUNITHOOK)GetProcAddress(hinstDll,"_SetRUNITHook@4");
	if(SetRUNITHook==0){
		FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_ALLOCATE_BUFFER,
			NULL,GetLastError(),MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
			(LPTSTR) &lpMsgBuf,0,NULL);
		MessageBox(GetActiveWindow(),lpMsgBuf,"RUNit LoadDLL-error SetRunitHook",0|MB_ICONERROR|MB_SETFOREGROUND);
		return FALSE;
	}
	SetSHIFT=(SETSHIFT)GetProcAddress(hinstDll,"SetSHIFT");
	if(SetSHIFT==0){
		FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_ALLOCATE_BUFFER,
			NULL,GetLastError(),MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
			(LPTSTR) &lpMsgBuf,0,NULL);
		MessageBox(GetActiveWindow(),lpMsgBuf,"RUNit LoadDLL-error SETShift",0|MB_ICONERROR|MB_SETFOREGROUND);
		return FALSE;
	}
	SetKEYBOARD=(SETKEYBOARD)GetProcAddress(hinstDll,"SetKEYBOARD");
	if(SetKEYBOARD==0){
		FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_ALLOCATE_BUFFER,
			NULL,GetLastError(),MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
			(LPTSTR) &lpMsgBuf,0,NULL);
		MessageBox(GetActiveWindow(),lpMsgBuf,"RUNit LoadDLL-error SETKeyboard",0|MB_ICONERROR|MB_SETFOREGROUND);
		return FALSE;
	}
	SetCRTLSHIFTSETUP=(SETCRTLSHIFTSETUP)GetProcAddress(hinstDll,"SetCRTLSHIFTSETUP");
	if(SetCRTLSHIFTSETUP==0){
		FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_ALLOCATE_BUFFER,
			NULL,GetLastError(),MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
			(LPTSTR) &lpMsgBuf,0,NULL);
		MessageBox(GetActiveWindow(),lpMsgBuf,"RUNit LoadDLL-error SETCrtlShiftSetup",0|MB_ICONERROR|MB_SETFOREGROUND);
		return FALSE;
	}

	SetHookMouse=(SETHOOKMOUSE)GetProcAddress(hinstDll,"SetHOOKMOUSE");
	if(SetHookMouse==0){
		FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_ALLOCATE_BUFFER,
			NULL,GetLastError(),MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
			(LPTSTR) &lpMsgBuf,0,NULL);
		MessageBox(GetActiveWindow(),lpMsgBuf,"RUNit LoadDLL-error SETHOOKMOUSE",0|MB_ICONERROR|MB_SETFOREGROUND);
		return FALSE;
	}


	//clear the RUNit struct
	ZeroMemory(&RUNit,sizeof(struct _RUNit));

	ReadFromIni(szMainIni,TRUE,0,FALSE);
	iFailed=0;
	for(i=1;i<RUNit.iNumSets;i++){
		if(!ReadFromIni(Set[i].szFile,FALSE,i-iFailed,FALSE)){
			strcpy(szSetError,"");
			strcpy(szSetError,"Problem while trying to read information for set #");
			itoa(i+1,sz,10);
			strcat(szSetError,sz);
			strcat(szSetError,".\n");
			strcat(szSetError,"the file '");
			strcat(szSetError,Set[i].szFile);
			strcat(szSetError,"' does not exist or contains no set information!");
			MessageBox(NULL,szSetError,"RUNit",MB_OK|MB_ICONERROR|MB_SETFOREGROUND);
			iFailed++;
		}
	}
	RUNit.iNumSets-=iFailed;
	RUNit.spaceLeft=8;
	RUNit.spaceMiddle=10;
	RUNit.spaceRight=40;
	
	//initialize menu-width
	//the width of the menu always seems to exceed than I set in WM_MEASUREITEM ...
	//...from what I understand it seems to be CXMENUSIZE+CXMENUCHECK...
	//but for safety reasons I put in another check in case WM_MEASUREITEM and Handle_WMDrawItem ...
	RUNit.iTooLarge=(GetSystemMetrics(SM_CXMENUSIZE)+GetSystemMetrics(SM_CXMENUCHECK));
	RUNit.iTooLargeSub=(GetSystemMetrics(SM_CXMENUCHECK));

	for (i=0;i<RUNit.iNumSets;i++)
		//ParseMenu(i);
		CreateIconCache(i);
	//////////////


	DoHooking();

	//accelerator
	//hAccel=LoadAccelerators(hInstance,MAKEINTRESOURCE(IDR_ACCELERATOR1));

	hWnd = CreateWindowEx(WS_EX_TOOLWINDOW,
			szAppName, szTitle, WS_CAPTION,1,1,200,600,NULL, NULL, hInstance, NULL);

	if (!hWnd) {
		return (FALSE);
	}

	hwndMain=hWnd;

	//tray icon
	nid.cbSize=sizeof(nid);
	nid.hWnd=hWnd;
	nid.uID=1234;
	nid.uFlags=NIF_ICON|NIF_MESSAGE|NIF_TIP;
	nid.uCallbackMessage=WM_APP+1;
	nid.hIcon=LoadIcon(hInst,MAKEINTRESOURCE(IDI_RUN));
	strcpy(nid.szTip,"Setup RUNit");
	if (!opt.bNoTray){
		Shell_NotifyIcon(NIM_ADD,&nid);
	}
	ShowWindow(hWnd,SW_HIDE);

	//god mode
	if (opt.bGodMode)
		MessageBox(GetDesktopWindow(),"Oh my god!\nI didn't think anybody would be so crazy to actually try this\n\nOK you've just entered GOD MODE\nyou can fly now and no bullet will hurt you during this session of RUNit","",MB_OK);

	//strcpy(Set[0].szName,"main");


	if (!Set[0].iApps){
		bDialogIsActive=TRUE;
		DialogBox(hInst, MAKEINTRESOURCE(IDD_REF), hWnd, (DLGPROC)QuickRefProc);	
		MessageBox(GetActiveWindow(),"RUNit - program list is empty!",
			APPNAME,MB_OK|MB_SETFOREGROUND|MB_ICONEXCLAMATION);
		bShouldExit=FALSE;
		bOK=DialogBox(hInst, MAKEINTRESOURCE(IDD_SETUP), NULL, (DLGPROC)SetupProc);
		if(bShouldExit){
			PostQuitMessage(0);
			return TRUE;
		}
		bDialogIsActive=FALSE;
		if(bOK){
			SetCursor(LoadCursor(NULL, IDC_WAIT));
			for (i=0;i<RUNit.iNumSets;i++)
				//ParseMenu(i);
				CreateIconCache(i);
			SetCursor(LoadCursor(NULL, IDC_ARROW));
		}


	}

	return (TRUE);
}

